<?php
/**
 * Set the password strenght requirements.
 * Author: Michael Finkenberger
 * @since V1.0.0.0
 * Last change in plugin version: V1.4.0.0
 * Date: 18.05.2021
 * Tested with plugin version: V1.4.0.0
 */

if(!defined('ABSPATH')) die();// no direct access



// Determine the password hint to explain the required complexity
function mfem_password_hint($hint) {
  
  return '<strong><u>'.__('Safety advice (brute-force attacks are increasing)','emergency-management').':</u></strong><br />'.mfem_get_pw_hint_text();
}
if ( get_option('mfem_pw_strength') ) add_filter( 'password_hint', 'mfem_password_hint', 10, 1 );



// Filter the password complexity to be in line with the requirements.
if ( get_option('mfem_pw_strength') ) {
  add_action( 'user_profile_update_errors', 'mfem_validateProfileUpdate', 10, 3 );
  add_filter( 'registration_errors', 'mfem_validateRegistration', 10, 3 );
  add_action( 'validate_password_reset', 'mfem_validatePasswordReset', 10, 2 );
}

function mfem_validateProfileUpdate( WP_Error &$errors, $update, &$user ) {
  global $you_cannot_reuse_your_old_password; // translated
  
  // check and complain if the new password equals the existing one
  // comparable code used as in function mfem_validate_password_change($errors,$user)
  $newpass1 = ( isset( $_POST[ 'pass1' ] ) && trim( $_POST[ 'pass1' ] ) ) ? $_POST[ 'pass1' ] : null;
  $existing_user = ( isset($user->ID) ) ? get_user_by('ID', $user->ID) : null;

  $mfem_user_roles = ( isset($existing_user->roles) ) ? (array)$existing_user->roles : array(); // Array with all roles the user has
  $mfem_expiries_array = (array)get_option('mfem_pw_expiry'); // Array with all expiring roles and their related expiry in days
  if ( count(array_intersect(array_keys($mfem_expiries_array), $mfem_user_roles)) != 0 ) { // If == 0: Expiry duration is not controlled for the user's role(s)
    $is_same = wp_check_password( $newpass1, $existing_user->data->user_pass, $user->ID );
    if ( $is_same ) {
      $errors->add( 'password_already_used', $you_cannot_reuse_your_old_password );
      return $errors;
    }
  }
  // and now check the password complexity
  return mfem_validateComplexPassword( $errors );
}

function mfem_validateRegistration( WP_Error $errors, $sanitized_user_login, $user_email ) {
  
  return mfem_validateComplexPassword( $errors );
}

function mfem_validatePasswordReset( WP_Error $errors, $userData ) {
  
  return mfem_validateComplexPassword( $errors );
}

function mfem_validateComplexPassword( $errors ) {
  
  $password = ( isset( $_POST[ 'pass1' ] ) && trim( $_POST[ 'pass1' ] ) ) ? $_POST[ 'pass1' ] : null;
  
// no password or already has password error
  if ( empty( $password ) || ( $errors->get_error_data( 'pass' ) ) )
    return $errors;
  
  // validate
  if ( ! mfem_isStrongPassword( $password ) )
    $errors->add( 'pass', '<strong>'.__('ERROR','emergency-management').'</strong>: '.__('Please note the following password rules','emergency-management').':<br />'.mfem_get_pw_hint_text() ); // Error message
  
  return $errors;
}

function mfem_isStrongPassword( $password ) {

  $mfem_pw_check = preg_match('/\s/',$password);
  if ( ( $mfem_pw_check === false ) || ( $mfem_pw_check <> 0 ) ) return false; // No white spaces
  
// Will be taken from the stored option 'mfem_pw_strength'
  $mfem_pw_strenght_store = get_option('mfem_pw_strength');
  $mfem_pw_content = $mfem_pw_strenght_store[0];
  $digit = ( in_array( 'digit', $mfem_pw_content) ) ? true : false;
  $uppercase = ( in_array( 'uppercase_char', $mfem_pw_content) ) ? true : false;
  $lowercase = ( in_array( 'lowercase_char', $mfem_pw_content) ) ? true : false;
  $special = ( in_array( 'special_char', $mfem_pw_content) ) ? true : false;
  $double = ( in_array( 'double_special', $mfem_pw_content) ) ? true : false;
  $pw_min = $mfem_pw_strenght_store[1];
  $pw_max = $mfem_pw_strenght_store[2];
  
  $pattern  = "/";
  if ($digit) $pattern .= "(?=.*[0-9])";
  if ($uppercase) $pattern .= "(?=.*[A-Z])";
  if ($lowercase) $pattern .= "(?=.*[a-z])";
  if ($special) $pattern .= "(?=.*[!\"#$%&'()*+,.\/:;<=>?@\[\\\\\]\^`{|}~])";
  if ($double) $pattern .= "(?=.*[!\"#$%&'()*+,.\/:;<=>?@\[\\\\\]\^`{|}~]{2})";
  $pattern .= ".{".$pw_min.",".$pw_max."}$/";
  return ( 1 === preg_match($pattern, $password) );
}



function mfem_get_pw_hint_text() {
  $mfem_special_chars = "! \" # $ % & ' ( ) * + , . / : ; < = > ? @ [ \ ] ^ ` { | } ~ ]";
  $mfem_pw_strenght_store = get_option('mfem_pw_strength');
  $mfem_pw_content = $mfem_pw_strenght_store[0];
  $digit = ( in_array( 'digit', $mfem_pw_content) ) ? '- '.__('digits','emergency-management').'<br />' : '';
  $uppercase = ( in_array( 'uppercase_char', $mfem_pw_content) ) ? '- '.__('uppercase letters','emergency-management').'<br />' : '';
  $lowercase = ( in_array( 'lowercase_char', $mfem_pw_content) ) ? '- '.__('lowercase letters','emergency-management').'<br />' : '';
  $special = ( in_array( 'special_char', $mfem_pw_content) ) ? '- '.__('special chars from','emergency-management').' '.$mfem_special_chars.'<br />' : '';
  $double = ( in_array( 'double_special', $mfem_pw_content) ) ? '- '.__('combined pairs of special chars from','emergency-management').' '.$mfem_special_chars.'<br />' : '';
  $pw_min = $mfem_pw_strenght_store[1];
  $pw_max = $mfem_pw_strenght_store[2];
  
  return __('The password needs to be between','emergency-management').' '.$pw_min.' '.__('and','emergency-management').' '.$pw_max.' '.
    __('characters in length and shall at least contain the following content','emergency-management').':<br />'.
    $digit.$uppercase.$lowercase.$special.$double.
    '- '.__('no spaces or whitespaces','emergency-management').'<br />'.
    '- '.__('Underscores (_) and hyphens (-) are allowed, but not considered as special chars','emergency-management').'.';
}


